'use strict';
angular.module('aom.services')
  .factory('aomJobDistanceSrv', function ($q, aomLocationSrv) {
    var jobDistance = {};

    function inRange(min, number, max) {
      if (!isNaN(number) && (number >= min) && (number <= max)) {
        return true;
      } else {
        return false;
      }
    }

    function valid_coords(number_lat, number_lng) {
      return (inRange(-90, number_lat, 90) && inRange(-180, number_lng, 180) && number_lat !== 0 && number_lng !== 0);
    }

    var toRad = function (num) {
      return num * Math.PI / 180;
    };

    function distanceBetween(start, end, options) {

      if (!valid_coords(start.latitude, start.longitude)) {
        return 5000;
      }

      if (!valid_coords(end.latitude, end.longitude)) {
        return 5000;
      }

      var km = 6371;
      var mile = 3960;
      options = options || {};

      var R = options.unit === 'mile' ? mile : km;

      var dLat = toRad(end.latitude - start.latitude);
      var dLon = toRad(end.longitude - start.longitude);
      var lat1 = toRad(start.latitude);
      var lat2 = toRad(end.latitude);

      var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.sin(dLon / 2) * Math.sin(dLon / 2) * Math.cos(lat1) * Math.cos(lat2);
      var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));

      if (options.threshold) {
        return options.threshold > (R * c);
      } else {
        return R * c;
      }
    }

    function distanceFromJob(job) {
      var distance = 5000;
      if (job.WorkLocation != null) {
        var workLocation = {
          latitude: parseFloat(job.WorkLocation.Latitude),
          longitude: parseFloat(job.WorkLocation.Longitude)
        };
        distance = distanceBetween(aomLocationSrv.data.lastKnownLocation, workLocation);
      }
      job.Distance = distance;
    }
    
    jobDistance.generateDistances = function (jobs) {
      var deferred = $q.defer();
      aomLocationSrv.getCurrentLocation(true)
        .then(function () {
          for (var i = 0; i < jobs.length; i++) {
            distanceFromJob(jobs[i]);
          }
          deferred.resolve();
        });
      return deferred.promise;
    };
    
    return jobDistance;
  });
