angular.module('aom.services')
  .factory('aomTrackingHistorySrv', function ($timeout, aomTrackingSrv, aomResourcesSrv, aomTrackingHelperSrv) {

    var trackingHistorySrv = {};

    trackingHistorySrv.data = {
      playingLocationHistory: false,
      locationPlayTimeout: null
    };

    function showUserRoutes() {
      var paths = {};
      aomTrackingSrv.data.layerBounds.locationHistory = [];
      for (var prop in aomTrackingSrv.data.modeState.selectedUser.history) {
        if (aomTrackingSrv.data.modeState.selectedUser.history.hasOwnProperty(prop)) {
          var element = aomTrackingSrv.data.modeState.selectedUser.history[prop];
          if (element.checked && element.Trips.length > 0) {
            for (var i = 0; i < element.Trips.length; i++) {
              paths[moment(element.Day).format('YYMMDD') + 'trip' + i] = {
                type: 'polyline',
                color: aomResourcesSrv.data.resources.Tracking.UserColours['colour05'], //hot pink
                weight: 3,
                opacity: 0.5,
                smoothFactor: 1,
                latlngs: element.Trips[i].LatitudeLongitudes
              };
              aomTrackingSrv.data.layerBounds.locationHistory = aomTrackingSrv.data.layerBounds.locationHistory.concat(element.Trips[i].latlngs);
            }
          }

        }
      }
      angular.copy(paths, aomTrackingSrv.data.paths);
    }

    trackingHistorySrv.positionLocationMarker = function (locationDay) {
      var scrollUnit = 10000 / (locationDay.Locations.length - 1);
      var index = Math.floor(locationDay.timePosition / scrollUnit);
      var location = locationDay.Locations[index];
      var message = aomResourcesSrv.data.resources.Tracking.DateIn + ' ' + moment(location.DateRecorded).format(aomResourcesSrv.data.resources.Formats.LongDateFormat);
      if (location.DateOut) {
        message += '.  ' + aomResourcesSrv.data.resources.Tracking.DateOut + ' ' + moment(location.DateOut).format(aomResourcesSrv.data.resources.Formats.LongDateFormat);
      }
      aomTrackingSrv.generateMarker('locationHistory', 'locationHistory', location.Latitude, location.Longitude, message, 1, '', '<i class="ion-ios-person user-icon"></i>');
    };

    trackingHistorySrv.showHistory = function (day, overlays) {
      var selectedUser = aomTrackingSrv.data.modeState.selectedUser;
      if (!day) {
        var latestDate = aomTrackingSrv.data.modeState.history ? Object.keys(selectedUser.history).pop() : moment().format('YYYYMMDD');
        day = selectedUser.history[latestDate];
      }
      aomTrackingSrv.hideSelectedUserHistory(overlays, selectedUser);
      if (day) {
        day.checked = true;
        overlays.locationHistory.visible = aomTrackingSrv.data.modeState.history;
      }
      showUserRoutes();

      if (day && aomTrackingSrv.data.modeState.history) {
        trackingHistorySrv.positionLocationMarker(day);
        var layer = day.Day + selectedUser.Id;
        if (!overlays[layer]) {
          overlays[layer] = aomTrackingSrv.createLayer(layer, true);
        }
        day.Jobs.map(function (obj) {
          aomTrackingSrv.generateMarker(day.Day + obj.id, layer, obj.WorkLocation.Latitude, obj.WorkLocation.Longitude, aomTrackingHelperSrv.jobTemplate(obj), 0.7, aomTrackingSrv.data.layerColours.jobHistory[layer], '');
          aomTrackingSrv.generateMarkerScope(day.Day + obj.id, obj);
        });
        overlays[layer].visible = true;
      }
    };

    trackingHistorySrv.stopLocationHistory = function () {
      $timeout.cancel(trackingHistorySrv.data.locationPlayTimeout);
      trackingHistorySrv.data.playingLocationHistory = false;
    };

    return trackingHistorySrv;
  });