'use strict';
angular.module('aom.controllers')
  .controller('SideMenuCtrl', function ($rootScope, $scope, $timeout, aomJobsSrv, aomResourcesSrv, aomCustomUriSrv, aomStateSrv, constants, aomDeviceSrv, $ionicSideMenuDelegate, aomScreenSizeSrv) {
    $scope.data = {
      resources: aomResourcesSrv.data.resources,
      sortOptions: {
        Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.SortOptions).where('$.Checked === true').toArray()[0].Text,
        SelectedParameterValue: aomResourcesSrv.data.resources.derived.SelectedSort,
        DisplayName: aomResourcesSrv.data.resources.Settings.SortJobListBy,
        Parameters: {
          Values: aomResourcesSrv.data.resources.appConfig.SortOptions
        }
      },
      jobsSrvData: aomJobsSrv.data,
      stateSrvData: aomStateSrv.data,
      viewStates: constants.viewStates,
      showMenu: true,
      showCloud: true,
      resultsNew: '',
      resultsPending: ''
    };

    adjustMenuWidth();

    window.addEventListener('resize', function () {
      $timeout(function () {
        if (getMenuStyle() === 'compact') {
          $scope.data.showCloud = window.innerHeight >= 375;
        } else {
          $scope.data.showCloud = window.innerHeight >= 555;
        }
      });
    }, false);

    window.addEventListener('orientationchange', function () {
      $timeout(function () {
        adjustMenuWidth();
      });
    }, false);

    function adjustMenuWidth() {
      $scope.data.showMenu = true;

      if ($ionicSideMenuDelegate.isOpenLeft())
        $ionicSideMenuDelegate.toggleLeft();

      var config = getMenuConfig();
      $scope.data.menuWidth = config.width;
      $scope.data.menuClass = config.class;
    }

    function displayMenu(show) {
      $timeout(() => {
        $scope.data.showMenu = show;
      });
    }

    function getMenuStyle() {
      var key = '';
      if (aomDeviceSrv.isTomTom()) {
        key = !aomDeviceSrv.isInPortrait() ? 'standard' : 'compact';
      } else {
        key = (!aomScreenSizeSrv.isTablet() && !aomDeviceSrv.isInPortrait()) ? 'compact' : 'standard';
      }
      return key;
    }

    function getMenuConfig() {
      return constants.menuSizes[getMenuStyle()];
    }

    if (aomDeviceSrv.isTomTom()) {
      window.addEventListener('native.keyboardshow', () => {
        if (aomDeviceSrv.isInPortrait()) displayMenu(false);
      });

      window.addEventListener('native.keyboardhide', () => {
        if (aomDeviceSrv.isInPortrait()) displayMenu(true);
      });
    } else if (!ionic.Platform.isIOS() &&
      ((!aomDeviceSrv.isInPortrait() && screen.height <= 767) || (aomDeviceSrv.isInPortrait() && screen.width <= 767))
    ) {
      window.addEventListener('native.keyboardshow', () => {
        if (!aomDeviceSrv.isInPortrait()) displayMenu(false);
      });

      window.addEventListener('native.keyboardhide', () => {
        if (!aomDeviceSrv.isInPortrait()) displayMenu(true);
      });
    }

    $scope.handleMenuClick = function handleMenuClick(menuItem) {
      if ($ionicSideMenuDelegate.isOpenLeft())
        $ionicSideMenuDelegate.toggleLeft();

      selectMenuItem(menuItem);
    };

    $scope.isSelectedMenuItem = function isSelectedMenuItem(currentMenuItem) {
      return currentMenuItem.toLowerCase() === aomStateSrv.data.selectedMenuItem;
    };

    $scope.showBadgeCount = function showBadgeCount(menuState) {
      return menuState === constants.viewStates.inbox || menuState === constants.viewStates.outbox;
    };

    $scope.getBadgeCount = function getBadgeCount(type) {
      var count = type === constants.viewStates.inbox
        ? getFilteredCount(aomJobsSrv.dataForJobs(), constants.status.new)
        : getFilteredCount(aomJobsSrv.dataForPendingJobs(), constants.status.pending);
      return count > 0 ? count : '';
    };

    function getFilteredCount(data, compareStatus) {
      return data.contents.filter(function (job) { return job.jsProperties.status === compareStatus; }).length;
    }

    function selectMenuItem(menuItem) {
      if ((aomDeviceSrv.isTomTom() || !aomScreenSizeSrv.isTablet()) && menuItem.state.main === aomStateSrv.data.main && menuItem.state.main === 'jobs') {
        $rootScope.$broadcast('backToJobs');
      }
      if (menuItem.state.main !== aomStateSrv.data.main) {
        aomStateSrv.data.showingJobMaps = false;
        aomStateSrv.setLeft(menuItem.state.main);
        aomStateSrv.setRight(menuItem.state.details);
        aomStateSrv.data.selectedMenuItemTitle = menuItem.title;
      }
    }

    aomCustomUriSrv.data.backState = 'app.Main';

    selectMenuItem(aomResourcesSrv.data.resources.v5.menu.Jobs);

    $scope.callClick = function (index, $event) {
      if ($event !== 'change') {
        $scope.selectObject.Parameters.Values[index].Checked = !$scope.selectObject.Parameters.Values[index].Checked;
      }

      if ($event === 'change' || $event.currentTarget.id === 'parent') {
        if (!$scope.multiSelect) {
          for (var i = 0; i < $scope.selectObject.Parameters.Values.length; i++) {
            $scope.selectObject.Parameters.Values[i].Checked = i === index ? $scope.selectObject.Parameters.Values[i].Checked : false;
          }
          if ($scope.selectObject.SelectedParameterValue !== undefined) {
            angular.copy($scope.selectObject.Parameters.Values[index], $scope.selectObject.SelectedParameterValue);
          }
        }
      }
    };

  });
